# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'puppet/property/boolean'

# Define the pg_cluster type
module Puppet
  Type.newtype(:pg_cluster) do
    include EasyType

    def self.module_name
      'pg_config'
    end

    desc <<-DOC
      This resource allows you create, manage and  delete a Postgres cluster.

          pg_cluster { '/database/db1':
            os_user => 'postgres',
          }

      A Postgres cluster is a location where Postgres stores the database content.
    DOC

    on_create do
      statement = [
        "#{actual_location}/bin/initdb",
        '--pgdata', name
      ]
      statement << '-X' << waldir if waldir
      statement << '--allow-group-access' if allow_group_access
      statement << "--username=#{db_user}" if db_user
      Puppet.debug Puppet::Util::Execution.execute(statement, :cwd => '/tmp', :uid => os_user, :gid => os_group, :failonfail => true, :combine => true)
    end

    on_destroy do
      FileUtils.rm_rf(name)
      FileUtils.rm_rf(waldir) if waldir
    end

    on_modify do
      fail 'At this point in time pg_cluster can NOT modify clusters'
    end

    on_prefetch do |resources, provider_class|
      resources.each do |name, properties|
        next unless File.exist?("#{name}/PG_VERSION")

        resource = { name => properties }.merge(:ensure => :present)
        resources[name].provider = provider_class.new(resource)
      end
    end

    def actual_location
      location.nil? ? "/usr/pgsql-#{first_found_version}" : location
    end

    def first_found_version
      # From version 17 downto 9
      17.downto(9).find { |v| File.exist?("/usr/pgsql-#{v}/bin/initdb") }
    end

    ensurable

    parameter :name
    parameter :location
    parameter :os_user
    parameter :os_group
    parameter :db_user
    parameter :allow_group_access
    parameter :waldir
  end
end
