# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'puppet_x/enterprisemodules/postgres/access'
require 'puppet_x/enterprisemodules/postgres/title_parser'

Puppet::Type.newtype(:pg_exec) do
  include EasyType
  include PuppetX::EnterpriseModules::Postgres::Access
  extend PuppetX::EnterpriseModules::Postgres::TitleParser

  desc <<-DOC
    This type allows you run a specific SQL statement or an sql file on a specified instance db.

        pg_exec{"drop table application_users/db@instance":
          role     => 'app_user',
          password => 'password,'
        }

    This statement will execute the sql statement `drop table application_users` on the database named `db` in the instance named `instance`.

    You can use the `unless` parameter to only execute the statement in certain states. If the query specified in the
    `unless` parameter returns one or more records, the main statement is skipped.

        pg_exec{ "create synonym ${user}.${synonym} for USER.${synonym}":
          unless  => "select * from all_synonyms where owner='${user}' and synonym_name='${synonym}'",
        }

    You can also execute a script.

        pg_exec{"@/tmp/do_some_stuff.sql/db@instance":
          role      => 'app_user',
          password  => 'password,'
          logoutput => on_failure,  # can be true, false or on_failure
        }

    This statement will run the sqlscript `/tmp/do_some_stuff.sql` on the db named `db`. Use the `unless`
    parameter to just execute the script in certain situations.

    When you don't specify the rolw     and the password, the type will connect as `sysdba`.


  DOC

  map_titles_to_attributes([
                             %r{^((@.*)/(.*)@(.*))$}, [:name, :statement, :db, :instance],
                             %r{^((.*)/(.*)@(.*))$},  [:name, :statement, :db, :instance],
                             %r{^((@.*)/(.*))$},      [:name, :statement, :db],
                             %r{^((.*)/(.*))$},       [:name, :statement, :db]
                           ])

  def refresh
    return unless provider.need_to_execute?

    provider.execute
    if parameters[:statement].sensitive
      Puppet.notice "#{path}: defined 'statement' as '[redacted]'"
    else
      Puppet.notice "#{path}: defined 'statement' as '#{statement}'"
    end
  end

  parameter :name
  property  :statement
  parameter :db
  parameter :instance

  parameter :timeout
  parameter :cwd
  parameter :logoutput
  parameter :password
  parameter :role
  parameter :unless
  parameter :onlyif
  parameter :refreshonly
  parameter :report_errors
  parameter :mark_as_error

  def onlyif_specified?
    !self[:onlyif].nil?
  end

  def unless_specified?
    !self[:unless].nil?
  end

  def report_errors?
    case self[:report_errors].to_s
    when 'true'
      true
    when 'false'
      false
    else
      fail 'invalid value for report_errors'
    end
  end
end
