# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
newproperty(:source) do
  include EasyType
  include PuppetX::EnterpriseModules::Postgres::Access

  desc <<-EOD
  A file describing the content of the function.

  Because puppet uses the contents of this file to decide if something needs to be done, the
  contents of this file must be very strict to the way Postgres stores the function source in the database.

  To decide if a change must be done, Puppet will fetch the function definition from the database and the contents
  of the source. For both it will remove all:

  - comments
  - empty lines
  - spaces
  - newlines
  - quotes

  and do a comparison. Te ensure correct detection of any differences, you do have to ensure the same order in your
  content as the function


  EOD

  data_type('Optional[String[1]]')

  to_translate_to_resource do |_raw_resource|
    'content-in-database'
  end

  def change_to_s(currentvalue, _newvalue)
    if currentvalue.to_s == 'absent'
      'created function definition'
    else
      'changed to new function definition'
    end
  end

  # rubocop: disable Naming/MethodParameterName
  def insync?(_is)
    if current_content == required_content
      true
    else
      current_content_file = Tempfile.new('pg_function')
      current_content_file.write(current_content)
      current_content_file.close
      required_content_file = Tempfile.new('pg_function')
      required_content_file.write(required_content)
      required_content_file.close
      binary_diff = `diff #{required_content_file.path} #{current_content_file.path}`
      Puppet.info "#{path}: Binary difference is:\n#{binary_diff}"
      false
    end
  end
  # rubocop: enable Naming/MethodParameterName

  def current_content
    return @current_content if @current_content

    bare_function_name = resource.function_name.split('(').first
    encoded_value = sql(template('pg_config/pg_function/function_definition.sql.erb', binding), :instance => resource[:instance], :db => resource.db, :parse => true).first['content']
    decoded_value = encoded_value.nil? ? nil : Array(encoded_value).pack('H*')
    @current_content ||= process_content(decoded_value)
  end

  def required_content
    @required_content ||= process_content(File.read(value))
  end

  def process_content(content)
    content.gsub(/CONTENT\s*\n-*\s*\n/, ''). \
      gsub(/--.*\n/, ''). \
      gsub(%r{^/\s*$}, ''). \
      gsub(/\s|\s|"|'/, ''). \
      delete("\n").upcase
  end
end
