# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#

require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift("#{Pathname.new(__FILE__).dirname.parent.parent.parent.parent}easy_typelib")
require 'easy_type'
require 'easy_type/array_property'
require 'puppet_x/enterprisemodules/postgres/access'
require 'puppet_x/enterprisemodules/postgres/title_parser'
require 'puppet_x/enterprisemodules/postgres/read_only_property'

Puppet::Type.newtype(:pg_parameter) do
  include EasyType
  include PuppetX::EnterpriseModules::Postgres::Access
  extend PuppetX::EnterpriseModules::Postgres::TitleParser

  desc <<-DOC
  This type allows you to manage the postgres parameters. Here is an example on how you can use this:

      pg_parameter {"jit_debugging_support@instance":
        value   => 'off',
      }

  The type will take care of setting the parameter settings and reloading the new configuration.
  DOC

  ensurable

  to_get_raw_resources do
    sql_on_cluster_databases "select name, encode(setting::bytea, 'base64') as setting, unit, vartype from pg_settings;"
  end

  on_create do
    fail "postgres parameter #{parameter_name}, does not exist."
  end

  on_modify do
    [:sql, "alter system set #{parameter_name} = #{specified_value}", { :instance => self[:instance], :db => 'postgres' }]
  end

  after_modify do
    sql('select pg_reload_conf()', :instance => self[:instance], :db => 'postgres')
  end

  on_destroy do
    fail "You cannot absent postgres parameter #{parameter_name}."
  end

  def specified_value
    fail 'pg_parameter must contain a value' unless self[:value]

    vartype_to_use = vartype || provider.vartype

    if %w[integer bool].include?(vartype_to_use)
      self[:value]
    else
      "'#{self[:value]}'"
    end
  end

  map_title_to_instance(:parameter_name)

  parameter :name
  parameter :parameter_name
  parameter :instance

  property  :value
  property  :vartype
  property  :unit
end
