#
# pg_config::pg_instance
#
# @summary This type manages a Postgres instance.
# It ensures the unit is created with the correct information and started. It also registers the database with Puppet so you can use all the other `pg_config` types on it.
#
# @param [Stdlib::Absolutepath] cluster
#    The Postgres clutsre you want the instance to run from.
#
# @param [Stdlib::Absolutepath] location
#    The location where to find the Postgres `/bin` directory.
#
# @param [String[1]] version
#    The version of Postgres the instance is running.
#
# @param [Enum['present', 'absent']] ensure
#    Ensure present or absent for the instance.
#
# @param [Enum['running', 'stopped']] status
#    The status of the service.
#
# @param [String[1]] os_user
#    The OS user you want the instance to run under.
#
# @param [String[1]] db_user
#    The database super user you want to use to connect to the database.
#
# @param [String[1]] os_group
#    The OS group you want the instance to run under.
#
# @param [Boolean] autostart
#    Autostart the database service after a reboot.
#    The default value is `true`.
#
# @param [Integer] port
#    The port you want the instance to run on.
#
#
# See the file "LICENSE" for the full license governing this code.
#
define pg_config::pg_instance (
  Stdlib::Absolutepath        $cluster,
  Stdlib::Absolutepath        $location,
  String[1]                   $version,
  Enum['present', 'absent']   $ensure    = 'present',
  Enum['running', 'stopped']  $status    = 'running',
  String[1]                   $os_user   = 'postgres',
  String[1]                   $os_group  = 'postgres',
  String[1]                   $db_user   = 'postgres',
  Boolean                     $autostart = true,
  Integer                     $port      = 5432
) {
  $service_name = if $title == 'default' {
    "postgresql-${version}.service"
  } else {
    "postgresql-${version}-${title}.service"
  }

  systemd::unit_file { $service_name:
    content => epp('pg_config/unit.service.epp', {
        version  => $version,
        os_user  => $os_user,
        os_group => $os_group,
        location => $location,
        cluster  => $cluster,
        port     => $port,
    }),
  }

  -> service { $service_name:
    ensure   => $status,
    enable   => $autostart,
    provider => 'systemd',
  }

  pg_instance { $title:
    os_user => $os_user,
    db_user => $db_user,
    port    => $port,
  }
}
